-- Document Generator
-- Reads Enhanced HTML and calls a handler for each tag that it encounters.
-- written by: Junko C. Miura (JCMiura@aol.com)
--             Rapid Deployment Software
--             http://members.aol.com/FilesEu/
-- Send problem reports to rds@attcanada.net

without warning
include docgen.e   -- common include file for RTF, HTML, and plain text
-- include rtf.e      -- output format: rich text
include html.e     -- output format: HTML
include text.e     -- output format: plain text

include wildcard.e
with warning

function param_list(object text)
-- read an HTMX tag and extract all the parameters and their values.
-- text must contain a '>' at the end, or text may be an atom.
-- format of text, before '>', when it is not an atom, should be:  
--           name1(=value1) name2(=value2) ...
--        where () means optional
    integer i, value_part
    sequence word, list
    
    if atom(text) then
	return {}
    end if
    
    list = {}
    i = 1
    value_part = FALSE
    while TRUE do
	while whitespace(text[i]) do
	    i = i + 1
	end while
	if text[i] = '>' then
	    exit
	end if
	if text[i] = '=' then
	    i = i + 1   
	    value_part = TRUE
	    while whitespace(text[i]) do
		i = i + 1
	    end while
	end if
	
	if text[i] = '"' then
	    -- read quoted-string parameter
	    word = ""
	    i = i + 1
	    while text[i] != '"' and i < length(text) do
		if text[i] = '\\' then
		    if text[i + 1] = '"' then
			word = word & text[i + 1]
		    else
			word = word & text[i] & text[i + 1]
		    end if
		    i = i + 2
		else
		    word = word & text[i]
		    i = i + 1
		end if
	    end while
	    if text[i] = '"' then
		i = i + 1
	    end if
	else 
	    word = ""
	    while not whitespace(text[i]) and text[i] != '=' and 
		  text[i] != '>' do
		word = word & text[i]
		i = i + 1
	    end while
	end if
	if value_part then
	    -- make a pair
	    list[length(list)] = {list[length(list)], word}
	    value_part = FALSE
	else
	    list = append(list, word)
	end if
    end while
    return list
end function

procedure tag(sequence name, object param, object raw)
-- process a tag (or literal text)
    integer r
    
    r = find(name, handler_name)
    if r = 0 then
	r = find("_default", handler_name)
	if r = 0 then
	    quit("Need a default handler")
	end if
    end if
    if equal(name, "!--") then
	call_proc(handler_id[r], {raw, ""})
    else
	call_proc(handler_id[r], {raw, param_list(param)})
    end if
end procedure

procedure scan()
    integer c, inDQuote
    sequence raw, name, param
    
    while TRUE do
	c = getc(in_file)
      
	if c = EOF then
	    exit
	elsif c = '<' then
	    -- an htmX tag?
	    c = getc(in_file)
	    if whitespace(c) or c = '=' then
		tag("_literal", '<', '<')
		tag("_literal", c, c)
	    else
		raw = "<"
		
		-- read the name
		name = ""
		while not whitespace(c) and c != '>' and c != EOF do
		    name = name & c
		    c = getc(in_file)
		end while
		name = lower(name)
		raw = raw & name
	    
		-- read the parameter list (if any)
		param = ""
		inDQuote = FALSE
		while not inDQuote and c != '>' and c != EOF do
		    if c = '"' then
			inDQuote = TRUE
			param = param & c
			c = getc(in_file)
			while inDQuote and c != EOF do
			    if c = '"' then
				inDQuote = FALSE
			    elsif c = '\\' then  
				-- control char. Just read in the next char.
				param = param & c
				c = getc(in_file) -- c might be EOF, watch...  
			    end if
			    param = param & c   -- so, c may be EOF, but ok
			    c = getc(in_file)   --   because it will be caught.
			end while
		    else
			param = param & c
			c = getc(in_file)
		    end if
		end while
	    
		if inDQuote then       -- checking inDQuote first makes sense
		    quit("double quote never closed") 
		elsif c = '>' then
		    param = param & '>'
		    raw = raw & param 
		    tag(name, param, raw)
		elsif c = EOF then
		    quit("premature EOF")
		end if
	    end if
	else
	    tag("_literal", c, c)
	end if
    end while
end procedure

procedure main()
    sequence cl, cl4
    sequence in_name, out_name
    integer p
    
    cl = command_line()
    if length(cl) <= 3 then
	quit("usage: ex doc filename option\n\n" &
	     "         where option may be one of\n" &
	     "               RTF or HTML or PLAIN")
    end if

    -- choose:
    cl4 = upper(cl[4])
    if equal(cl4, "RTF") then
	abort(1)
	-- rtf_init()
    elsif equal(cl4, "HTML") then
	html_init()
    else         -- default is to produce plain text formated document
	text_init()
    end if
    
    in_name = upper(cl[3])
    if not match(".HTX", in_name) then
	quit("must be a .htx file")
    end if
    in_file = open(in_name, "r")
    if in_file = -1 then
	quit("couldn't open " & in_name)
    end if
    p = find('.', in_name)
    out_name = in_name[1..p] & out_type
    out_file = open(out_name, "w")
    if out_file = -1 then
	quit("couldn't open " & out_name)
    end if

    scan()
    
    if equal(cl4, "PLAIN") then
	text_end()
    end if
    close(in_file)
    close(out_file)
end procedure

main()

